package com.jiuqi.syntax;

package .tmp;

import com.jiuqi.syntax.CommonData;
import com.jiuqi.syntax.DataCell;
import com.jiuqi.syntax.NiporlanItem;
import java.io.PrintStream;
import java.util.HashMap;

public class NiporlanItemFactory {
  public PrintStream logger = null;
  
  private long id = System.currentTimeMillis();
  
  private static com.jiuqi.syntax.NiporlanItemFactory globalFactory = new com.jiuqi.syntax.NiporlanItemFactory();
  
  private static boolean useGlobalFactory = true;
  
  public static boolean getUseGlobalFactory() {
    return useGlobalFactory;
  }
  
  public static void setUseGlobalFactory(boolean value) {
    useGlobalFactory = value;
  }
  
  public static com.jiuqi.syntax.NiporlanItemFactory getFactory() {
    if (useGlobalFactory)
      return globalFactory; 
    return new com.jiuqi.syntax.NiporlanItemFactory();
  }
  
  protected static HashMap OP_NIS = new HashMap<Object, Object>();
  
  protected HashMap FUNC_NIS;
  
  protected static NiporlanItem BOOL_TRUE_NI;
  
  protected static NiporlanItem BOOL_FALSE_NI;
  
  protected static NiporlanItem BOOL_NULL_NI;
  
  public int FOUND_COUNT;
  
  public int MISSED_COUNT;
  
  public int NULL_COUNT;
  
  protected HashMap[] dataCellCache;
  
  static {
    for (int i = 1; i <= 20; i++)
      OP_NIS.put(new Integer(i), new NiporlanItem(i)); 
    CommonData cd = new CommonData();
    cd.type = 5;
    cd.isNull = false;
    cd.cdBool = true;
    BOOL_TRUE_NI = new NiporlanItem(cd);
    cd = new CommonData();
    cd.type = 5;
    cd.isNull = false;
    cd.cdBool = false;
    BOOL_FALSE_NI = new NiporlanItem(cd);
    cd = new CommonData();
    cd.type = 5;
    cd.isNull = true;
    BOOL_NULL_NI = new NiporlanItem(cd);
  }
  
  public NiporlanItem getNiporlanItem(int opCode) {
    NiporlanItem ni = null;
    synchronized (OP_NIS) {
      ni = (NiporlanItem)OP_NIS.get(new Integer(opCode));
      if (ni == null) {
        ni = new NiporlanItem(opCode);
        OP_NIS.put(new Integer(opCode), ni);
        if (this.logger != null)
          this.logger.println(String.valueOf(Thread.currentThread().getName()) + "\t" + "添加新的操作符(" + opCode + ")共享NiporlanItem "); 
      } 
    } 
    return ni;
  }
  
  public NiporlanItemFactory() {
    this.FUNC_NIS = new HashMap<Object, Object>();
    this.FOUND_COUNT = 0;
    this.MISSED_COUNT = 0;
    this.NULL_COUNT = 0;
    this.dataCellCache = null;
    if (this.logger != null)
      this.logger.println(String.valueOf(Thread.currentThread().getName()) + "\t" + "Crate NO." + this.id + " NIFactory."); 
  }
  
  public NiporlanItem getNiporlanItem(String funcName) {
    if (funcName == null)
      return null; 
    NiporlanItem ni = null;
    synchronized (this.FUNC_NIS) {
      ni = (NiporlanItem)this.FUNC_NIS.get(funcName);
      if (ni == null) {
        ni = new NiporlanItem(funcName);
        this.FUNC_NIS.put(funcName, ni);
        if (this.logger != null)
          this.logger.println(String.valueOf(Thread.currentThread().getName()) + "\t" + "添加新的函数(" + funcName + ")共享NiporlanItem "); 
      } 
    } 
    return ni;
  }
  
  public NiporlanItem getNiporlanItem(CommonData commonData) {
    if (commonData != null && commonData.type == 5) {
      if (commonData.isNull)
        return BOOL_NULL_NI; 
      if (commonData.cdBool)
        return BOOL_TRUE_NI; 
      if (!commonData.cdBool)
        return BOOL_FALSE_NI; 
    } 
    return new NiporlanItem(commonData);
  }
  
  protected static int PAGE_COUNT = 2000;
  
  public NiporlanItem getNiporlanItem(DataCell dataCell) {
    if (this.dataCellCache == null) {
      this.dataCellCache = new HashMap[PAGE_COUNT];
      for (int i = 0; i < this.dataCellCache.length; i++)
        this.dataCellCache[i] = new HashMap<Object, Object>(); 
      if (this.logger != null)
        this.logger.println(String.valueOf(Thread.currentThread().getName()) + "\t" + "DataCell缓冲池初始化 " + System.currentTimeMillis()); 
    } 
    if (dataCell == null) {
      if (this.logger != null)
        this.logger.println(" DataCell is null"); 
      return new NiporlanItem(dataCell);
    } 
    HashMap<String, DataCell> page = null;
    int index = dataCell.tableOffset;
    if (index < 0)
      index = -index; 
    page = this.dataCellCache[index % PAGE_COUNT];
    DataCell oldDataCell = null;
    String mapKey = dataCell.toString();
    oldDataCell = (DataCell)page.get(mapKey);
    if (oldDataCell == null)
      synchronized (page) {
        if (this.logger != null)
          this.logger.println(String.valueOf(Thread.currentThread().getName()) + "\t" + "未命中 " + mapKey); 
        oldDataCell = (DataCell)page.get(mapKey);
        if (oldDataCell == null) {
          if (this.logger != null)
            this.logger.println(String.valueOf(Thread.currentThread().getName()) + "\t" + "连续未命中 " + mapKey); 
          DataCell newDataCell = new DataCell();
          newDataCell.assign(dataCell);
          page.put(mapKey, newDataCell);
        } 
      }  
    if (oldDataCell != null && 
      oldDataCell.statItem == dataCell.statItem && 
      oldDataCell.offset == dataCell.offset)
      return new NiporlanItem(oldDataCell); 
    return new NiporlanItem(dataCell);
  }
  
  public NiporlanItem getNiporlanItem(Object obj) {
    return new NiporlanItem(obj);
  }
  
  public String getProductCountDesc() {
    return "Factory NO." + this.id;
  }
  
  public void destroy() {
    if (!useGlobalFactory && this != globalFactory)
      this.FUNC_NIS.clear(); 
  }
}
